use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Anwar
Burke
Caleb
Cushing
Custodio
David
Fredric
Golden
Graham
Kent
Knop
Mengué
Mohammad
Olivier
Paulo
Pod
Sean
Spell
Wordlist
bin
dagolden
dolmen
haarg
kentfredric
lib
mohammad
pauloscustodio
podspell
sburke
xenoterracide
